<?php

// Include external locations file
include 'locations.php';

// Global variable for location data
global $locations;

/**
 * Function to find the closest match for Arabic text and gender, returning the location ID.
 *
 * @param string $inputArabic  Arabic text input.
 * @param string $gender       Gender filter ("Male" or "Female").
 * @param array  $locations    Array of location data.
 *
 * @return string|null         Returns location ID if found, else null.
 */
function findClosestLocation($inputArabic, $gender) {
    global $locations;
    $closestLocation = null;
    $shortestDistance = null;

    foreach ($locations as $location) {
        // Filter by gender
        if (strcasecmp($location['gender'], $gender) !== 0) {
            continue;
        }

        // Calculate the Levenshtein distance between input and current Arabic name
        $distance = levenshtein($inputArabic, $location['arabic_name']);

        if ($shortestDistance === null || $distance < $shortestDistance) {
            $shortestDistance = $distance;
            $closestLocation = $location['location_id'];
        }
    }

    return $closestLocation;
}

/**
 * Establish SQLite database connection.
 *
 * @return SQLite3 Database connection instance.
 */
function dbx() {
    $db = new SQLite3('mydatas.db');
    
    // Create necessary tables if they don't exist
    $db->exec('CREATE TABLE IF NOT EXISTS data_logs (id INTEGER PRIMARY KEY, time TEXT, json TEXT)');
    
    $createTableSQL = "
        CREATE TABLE IF NOT EXISTS locations (
            locationId INTEGER PRIMARY KEY AUTOINCREMENT,
            locationCode TEXT NOT NULL,
            locationName TEXT NOT NULL
        );
    ";
    $db->exec($createTableSQL);
    
    return $db;
}

/**
 * Check if a location exists in the database by its code or name.
 *
 * @param SQLite3 $db Database connection.
 * @param string  $searchValue The location code or name to search.
 *
 * @return bool True if location exists, otherwise false.
 */
function locationExists($db, $searchValue) {
    $query = "SELECT COUNT(*) as count FROM locations WHERE locationCode = :searchValue OR locationName = :searchValue";
    $stmt = $db->prepare($query);
    $stmt->bindValue(':searchValue', $searchValue, SQLITE3_TEXT);

    $result = $stmt->execute();
    $row = $result->fetchArray(SQLITE3_ASSOC);

    return $row['count'] > 0;
}
/**
 * Function to delete all locations from the database.
 *
 * @param SQLite3 $db Database connection.
 * @return bool True if locations were deleted successfully, false otherwise.
 */
function deleteAllLocations($db) {
    // Delete all rows from the locations table
    $query = "DELETE FROM locations";
    $result = $db->exec($query);

    // Check if the deletion was successful
    if ($result) {
        echo "All locations have been deleted successfully.\n";
        return true;
    } else {
        echo "Failed to delete locations.\n";
        return false;
    }
}

/**
 * Function to get the Location ID from the database by location name
 *
 * @param string $locationName The name of the location
 * @param SQLite3 $db Database connection
 * @return int|null The location ID if found, otherwise null
 */
function getLocationIdByName($locationName, $db) {
    // Prepare the SQL query to find the locationId based on location name
    $query = "SELECT locationId FROM locations WHERE locationName = :locationName LIMIT 1";
    $stmt = $db->prepare($query);
    $stmt->bindValue(':locationName', $locationName, SQLITE3_TEXT);

    // Execute the query and fetch the result
    $result = $stmt->execute();
    $location = $result->fetchArray(SQLITE3_ASSOC);

    // If a matching location is found, return the location ID
    if ($location) {
        return $location['locationId'];
    }

    // If no location is found, return null
    return null;
}


/**
 * Insert a location into the database if it does not already exist.
 *
 * @param SQLite3 $db Database connection.
 * @param int $locationId Location ID.
 * @param string $locationCode Location code.
 * @param string $locationName Location name.
 *
 * @return bool True if the location was inserted, otherwise false.
 */
function insertLocationIfNotExists($db, $locationId, $locationCode, $locationName) {
    $checkQuery = "SELECT COUNT(*) as count , locations.* FROM locations 
                   WHERE locationCode = :locationCode";
    $stmt = $db->prepare($checkQuery);
    // $stmt->bindValue(':locationId', $locationId, SQLITE3_INTEGER);
    $stmt->bindValue(':locationCode', $locationCode, SQLITE3_TEXT);
    // $stmt->bindValue(':locationName', $locationName, SQLITE3_TEXT);
    
    $result = $stmt->execute();
    $row = $result->fetchArray(SQLITE3_ASSOC);

    if ($row['count'] > 0) {
        var_dump($row);
        echo "Location already exists: ID - $locationId, Code - $locationCode, Name - $locationName\n";
        return false;
    }

    $insertQuery = "INSERT INTO locations (locationId, locationCode, locationName) VALUES (:locationId, :locationCode, :locationName)";
    $insertStmt = $db->prepare($insertQuery);
    $insertStmt->bindValue(':locationId', $locationId, SQLITE3_INTEGER);
    $insertStmt->bindValue(':locationCode', $locationCode, SQLITE3_TEXT);
    $insertStmt->bindValue(':locationName', $locationName, SQLITE3_TEXT);
    
    $insertStmt->execute();
    echo "Location inserted successfully: ID - $locationId, Code - $locationCode, Name - $locationName\n";
    
    return true;
}

/**
 * Retrieve location information by its code or name.
 *
 * @param SQLite3 $db Database connection.
 * @param string  $searchValue The location code or name.
 *
 * @return array|null Location data or null if not found.
 */
function getLocation($db, $searchValue) {
    $query = "SELECT * FROM locations WHERE locationCode = :searchValue";
    $stmt = $db->prepare($query);
    $stmt->bindValue(':searchValue', $searchValue, SQLITE3_TEXT);
    
    $result = $stmt->execute();
    $location = $result->fetchArray(SQLITE3_ASSOC);
    
    if ($location) {
        return $location;
    }
    
    return null;
}



// add lead
function addLead($arr){
    // xx($arr);
    // var_dump($arr);
    $json = json_encode($arr);
    $curl = curl_init();

    curl_setopt_array($curl, array(
    CURLOPT_URL => 'https://uatin2fitnessapi.sentinelscope.com/api/Sentinel/AddLead',
    CURLOPT_RETURNTRANSFER => true,
    CURLOPT_ENCODING => '',
    CURLOPT_MAXREDIRS => 10,
    CURLOPT_TIMEOUT => 0,
    CURLOPT_FOLLOWLOCATION => true,
    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
    CURLOPT_CUSTOMREQUEST => 'POST',
    CURLOPT_POSTFIELDS => $json,
    CURLOPT_HTTPHEADER => array(
        'Content-Type: application/json',
        'Authorization: Bearer '.getToken()
        ),
    ));

    $response = curl_exec($curl);

    // var_dump($arr);
    curl_close($curl);
    if($response === false){
        return false;
    }
    return json_decode($response, true);
}

/**
 * Fetch the token required for making API requests.
 *
 * @return string|null The access token or null if the request failed.
 */
function getToken() {
    $curl = curl_init();
    
    curl_setopt_array($curl, [
        CURLOPT_URL => 'https://uatin2fitnessapi.sentinelscope.com/Token',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTPHEADER => ['Content-Type: application/x-www-form-urlencoded'],
        CURLOPT_POSTFIELDS => 'grant_type=password&username=testing&password=Test%40123',
    ]);
    
    $response = curl_exec($curl);
    curl_close($curl);
    
    if ($response === false) {
        return null;
    }
    
    $responseData = json_decode($response, true);
    // var_dump($responseData);
    return $responseData['access_token'] ?? null;
}

/**
 * Fetch location data from an external API.
 *
 * @return array The list of locations with codes and IDs.
 */
function fetchLocationData() {
    $curl = curl_init();
    
       curl_setopt_array($curl, array(
    CURLOPT_URL => 'https://uatin2fitnessapi.sentinelscope.com/api/Sentinel/AddLeadInitialModel',
    CURLOPT_RETURNTRANSFER => true,
    CURLOPT_ENCODING => '',
    CURLOPT_MAXREDIRS => 10,
    CURLOPT_TIMEOUT => 0,
    CURLOPT_FOLLOWLOCATION => true,
    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
    CURLOPT_CUSTOMREQUEST => 'POST',
    CURLOPT_POSTFIELDS => '',
    CURLOPT_HTTPHEADER => array(
        'Content-Type: application/json',
        'Authorization: Bearer '.getToken()
        ),
    ));
    
    $response = curl_exec($curl);
    curl_close($curl);
    
    $responseData = json_decode($response, true);
    // var_dump($responseData);
    return $responseData['Result']['LocationList'] ?? [];
}

function extractCountryCodeAndNumber($phoneNumber) {
    // Remove any spaces, dashes, or parentheses from the number
    $cleanedNumber = preg_replace('/[^\d]/', '', $phoneNumber);

   

// data from https://gist.github.com/andyj/7108917

$countryCodes = [
	'44' => 'UK (+44)',
	'1' => 'USA (+1)',
	'213' => 'Algeria (+213)',
	'376' => 'Andorra (+376)',
	'244' => 'Angola (+244)',
	'1264' => 'Anguilla (+1264)',
	'1268' => 'Antigua & Barbuda (+1268)',
	'54' => 'Argentina (+54)',
	'374' => 'Armenia (+374)',
	'297' => 'Aruba (+297)',
	'61' => 'Australia (+61)',
	'43' => 'Austria (+43)',
	'994' => 'Azerbaijan (+994)',
	'1242' => 'Bahamas (+1242)',
	'973' => 'Bahrain (+973)',
	'880' => 'Bangladesh (+880)',
	'1246' => 'Barbados (+1246)',
	'375' => 'Belarus (+375)',
	'32' => 'Belgium (+32)',
	'501' => 'Belize (+501)',
	'229' => 'Benin (+229)',
	'1441' => 'Bermuda (+1441)',
	'975' => 'Bhutan (+975)',
	'591' => 'Bolivia (+591)',
	'387' => 'Bosnia Herzegovina (+387)',
	'267' => 'Botswana (+267)',
	'55' => 'Brazil (+55)',
	'673' => 'Brunei (+673)',
	'359' => 'Bulgaria (+359)',
	'226' => 'Burkina Faso (+226)',
	'257' => 'Burundi (+257)',
	'855' => 'Cambodia (+855)',
	'237' => 'Cameroon (+237)',
	'1' => 'Canada (+1)',
	'238' => 'Cape Verde Islands (+238)',
	'1345' => 'Cayman Islands (+1345)',
	'236' => 'Central African Republic (+236)',
	'56' => 'Chile (+56)',
	'86' => 'China (+86)',
	'57' => 'Colombia (+57)',
	'269' => 'Comoros (+269)',
	'242' => 'Congo (+242)',
	'682' => 'Cook Islands (+682)',
	'506' => 'Costa Rica (+506)',
	'385' => 'Croatia (+385)',
	'53' => 'Cuba (+53)',
	'90392' => 'Cyprus North (+90392)',
	'357' => 'Cyprus South (+357)',
	'42' => 'Czech Republic (+42)',
	'45' => 'Denmark (+45)',
	'253' => 'Djibouti (+253)',
	'1809' => 'Dominica (+1809)',
	'1809' => 'Dominican Republic (+1809)',
	'593' => 'Ecuador (+593)',
	'20' => 'Egypt (+20)',
	'503' => 'El Salvador (+503)',
	'240' => 'Equatorial Guinea (+240)',
	'291' => 'Eritrea (+291)',
	'372' => 'Estonia (+372)',
	'251' => 'Ethiopia (+251)',
	'500' => 'Falkland Islands (+500)',
	'298' => 'Faroe Islands (+298)',
	'679' => 'Fiji (+679)',
	'358' => 'Finland (+358)',
	'33' => 'France (+33)',
	'594' => 'French Guiana (+594)',
	'689' => 'French Polynesia (+689)',
	'241' => 'Gabon (+241)',
	'220' => 'Gambia (+220)',
	'7880' => 'Georgia (+7880)',
	'49' => 'Germany (+49)',
	'233' => 'Ghana (+233)',
	'350' => 'Gibraltar (+350)',
	'30' => 'Greece (+30)',
	'299' => 'Greenland (+299)',
	'1473' => 'Grenada (+1473)',
	'590' => 'Guadeloupe (+590)',
	'671' => 'Guam (+671)',
	'502' => 'Guatemala (+502)',
	'224' => 'Guinea (+224)',
	'245' => 'Guinea - Bissau (+245)',
	'592' => 'Guyana (+592)',
	'509' => 'Haiti (+509)',
	'504' => 'Honduras (+504)',
	'852' => 'Hong Kong (+852)',
	'36' => 'Hungary (+36)',
	'354' => 'Iceland (+354)',
	'91' => 'India (+91)',
	'62' => 'Indonesia (+62)',
	'98' => 'Iran (+98)',
	'964' => 'Iraq (+964)',
	'353' => 'Ireland (+353)',
	'972' => 'Israel (+972)',
	'39' => 'Italy (+39)',
	'1876' => 'Jamaica (+1876)',
	'81' => 'Japan (+81)',
	'962' => 'Jordan (+962)',
	'7' => 'Kazakhstan (+7)',
	'254' => 'Kenya (+254)',
	'686' => 'Kiribati (+686)',
	'850' => 'Korea North (+850)',
	'82' => 'Korea South (+82)',
	'965' => 'Kuwait (+965)',
	'996' => 'Kyrgyzstan (+996)',
	'856' => 'Laos (+856)',
	'371' => 'Latvia (+371)',
	'961' => 'Lebanon (+961)',
	'266' => 'Lesotho (+266)',
	'231' => 'Liberia (+231)',
	'218' => 'Libya (+218)',
	'417' => 'Liechtenstein (+417)',
	'370' => 'Lithuania (+370)',
	'352' => 'Luxembourg (+352)',
	'853' => 'Macao (+853)',
	'389' => 'Macedonia (+389)',
	'261' => 'Madagascar (+261)',
	'265' => 'Malawi (+265)',
	'60' => 'Malaysia (+60)',
	'960' => 'Maldives (+960)',
	'223' => 'Mali (+223)',
	'356' => 'Malta (+356)',
	'692' => 'Marshall Islands (+692)',
	'596' => 'Martinique (+596)',
	'222' => 'Mauritania (+222)',
	'269' => 'Mayotte (+269)',
	'52' => 'Mexico (+52)',
	'691' => 'Micronesia (+691)',
	'373' => 'Moldova (+373)',
	'377' => 'Monaco (+377)',
	'976' => 'Mongolia (+976)',
	'1664' => 'Montserrat (+1664)',
	'212' => 'Morocco (+212)',
	'258' => 'Mozambique (+258)',
	'95' => 'Myanmar (+95)',
	'264' => 'Namibia (+264)',
	'674' => 'Nauru (+674)',
	'977' => 'Nepal (+977)',
	'31' => 'Netherlands (+31)',
	'687' => 'New Caledonia (+687)',
	'64' => 'New Zealand (+64)',
	'505' => 'Nicaragua (+505)',
	'227' => 'Niger (+227)',
	'234' => 'Nigeria (+234)',
	'683' => 'Niue (+683)',
	'672' => 'Norfolk Islands (+672)',
	'670' => 'Northern Marianas (+670)',
	'47' => 'Norway (+47)',
	'968' => 'Oman (+968)',
	'680' => 'Palau (+680)',
	'507' => 'Panama (+507)',
	'675' => 'Papua New Guinea (+675)',
	'595' => 'Paraguay (+595)',
	'51' => 'Peru (+51)',
	'63' => 'Philippines (+63)',
	'48' => 'Poland (+48)',
	'351' => 'Portugal (+351)',
	'1787' => 'Puerto Rico (+1787)',
	'974' => 'Qatar (+974)',
	'262' => 'Reunion (+262)',
	'40' => 'Romania (+40)',
	'7' => 'Russia (+7)',
	'250' => 'Rwanda (+250)',
	'378' => 'San Marino (+378)',
	'239' => 'Sao Tome & Principe (+239)',
	'966' => 'Saudi Arabia (+966)',
	'221' => 'Senegal (+221)',
	'381' => 'Serbia (+381)',
	'248' => 'Seychelles (+248)',
	'232' => 'Sierra Leone (+232)',
	'65' => 'Singapore (+65)',
	'421' => 'Slovak Republic (+421)',
	'386' => 'Slovenia (+386)',
	'677' => 'Solomon Islands (+677)',
	'252' => 'Somalia (+252)',
	'27' => 'South Africa (+27)',
	'34' => 'Spain (+34)',
	'94' => 'Sri Lanka (+94)',
	'290' => 'St. Helena (+290)',
	'1869' => 'St. Kitts (+1869)',
	'1758' => 'St. Lucia (+1758)',
	'249' => 'Sudan (+249)',
	'597' => 'Suriname (+597)',
	'268' => 'Swaziland (+268)',
	'46' => 'Sweden (+46)',
	'41' => 'Switzerland (+41)',
	'963' => 'Syria (+963)',
	'886' => 'Taiwan (+886)',
	'7' => 'Tajikstan (+7)',
	'66' => 'Thailand (+66)',
	'228' => 'Togo (+228)',
	'676' => 'Tonga (+676)',
	'1868' => 'Trinidad & Tobago (+1868)',
	'216' => 'Tunisia (+216)',
	'90' => 'Turkey (+90)',
	'7' => 'Turkmenistan (+7)',
	'993' => 'Turkmenistan (+993)',
	'1649' => 'Turks & Caicos Islands (+1649)',
	'688' => 'Tuvalu (+688)',
	'256' => 'Uganda (+256)',
	'380' => 'Ukraine (+380)',
	'971' => 'United Arab Emirates (+971)',
	'598' => 'Uruguay (+598)',
	'7' => 'Uzbekistan (+7)',
	'678' => 'Vanuatu (+678)',
	'379' => 'Vatican City (+379)',
	'58' => 'Venezuela (+58)',
	'84' => 'Vietnam (+84)',
	'84' => 'Virgin Islands - British (+1284)',
	'84' => 'Virgin Islands - US (+1340)',
	'681' => 'Wallis & Futuna (+681)',
	'969' => 'Yemen (North)(+969)',
	'967' => 'Yemen (South)(+967)',
	'260' => 'Zambia (+260)',
	'263' => 'Zimbabwe (+263)',
];
    $countryCodes = array_keys($countryCodes);

    // Find the matching country code
    foreach ($countryCodes as $code) {
        if (strpos($cleanedNumber, $code) === 0) {
            $localNumber = substr($cleanedNumber, strlen($code));
            return [
                'countryCode' => $code,
                'localNumber' => $localNumber
            ];
        }
    }

    // If no country code is found, assume the whole number is local
    return [
        'countryCode' => null,
        'localNumber' => $cleanedNumber
    ];
}


 

/**
 * Function to process and transform the JSON data into structured format
 *
 * @param array $jsonData Decoded JSON data
 * @param array $locations Array of location data to find the closest match
 * @return array Transformed data with locationid
 */
function transformJsonDataWithLocationId($jsonData) {
    global $db;
    $newData = [];

    // Loop through each JSON item
    foreach ($jsonData as $d) {
        // Split full_name into FirstName and LastName
        $name = explode(' ', $d['full_name']);
        $newData['FirstName'] = $name[0];
        $newData['LastName'] = substr($d['full_name'], strlen($name[0]));

        // Extract Gender, Email, Campaign, Fitness, and Phone Number fields
        $newData['Gender'] = $d['gender'] ?? '';
        $newData['Email'] = $d['email'] ?? '';
        $newData['Campaign'] = $d['cam_name'] ?? '';
        $newData['NearestClub'] = $d['fitness'] ?? '';
        
        $newData['DateOfBirth'] = '';
        
        $newData['LegalId'] = '';
        

        $newData['MarketingSourceId'] = 1;
        $newData['MarketingSource'] = 'facebook';
        $newData['FullPhone'] = $d['phone_number'];


        // Find the location ID based on the Arabic fitness name
        $locationId = findClosestLocation($d['fitness'], $d['gender']);
        if ($locationId) {
            // code to id
            $locationData = getLocation($db, $locationId);
            // var_dump($locationID);
            if ($locationData) {
                      $newData['LocationId'] = ($locationData['locationId']);
                
                // echo ' exist';
            }else{
                // fetch again or null
                
            $newData['LocationId'] = null;
            
                
            }
        } else {
            $newData['LocationId'] = null; // If no location is found
        }
        // if()
        if (strpos($d['phone_number'], '+') !== false) {
            $result = extractCountryCodeAndNumber($d['phone_number']); 

            if ($result) {
                $newData['CountryCode']  =  $result['countryCode'] ;
                $d['phone_number'] = $result['localNumber']; 
    
                
            }else{
                
            
            $d['phone_number'] =  str_replace(['+966'],[''],$d['phone_number']);
            
            $newData['CountryCode'] = '966';
                
            }
            
            
            $newData['Mobile'] =  $d['phone_number'];
            // have code
        }else{ 
            // no codes
        $d['phone_number'] =  $d['phone_number'];
        // Extract country code and mobile number from the phone number
        $newData['CountryCode'] = '966';
        $newData['Mobile'] =  $d['phone_number'];
        }
        // // Clean up the phone number to remove non-numeric characters
        // $d['phone_number'] = str_replace(['+', ' '], '', $d['phone_number']);
        // // Extract country code and mobile number from the phone number
        // $newData['CountryCode'] = substr($d['phone_number'], 0, 3);
        // $newData['Mobile'] =  $d['phone_number'];
    }

    return $newData;
}



// Main application logic
// global $db;
// $db = db();
// // deleteAllLocations($db);
// // Pagination parameters
// $page = $_GET['page'] ?? 1;
// $limit = $_GET['limit'] ?? 5;
// $offset = ($page - 1) * $limit;

// // Retrieve data logs with pagination
// $res = $db->query('SELECT * FROM data_logs LIMIT ' . $limit . ' OFFSET ' . $offset);
// $notFoundLocations = [];
// $locationsList = fetchLocationData();
// // var_dump($locationsList);

// $locationCodeToId = array_column($locationsList, 'LocationId', 'LocationCode');

// // Process each data log
// while ($row = $res->fetchArray(SQLITE3_ASSOC)) {
//     $json = json_decode($row['json'], true);
    
//     $payload = (transformJsonDataWithLocationId($json));
//   $data=  addLead($payload);
//   var_dump($data);
//     // exit; 
// }

// // Output unmatched locations
// // echo "Unmatched locations: " . implode(', ', $notFoundLocations) . "\n";

// // Total count of data logs
// // $res = $db->query('SELECT COUNT(*) as total FROM data_logs');
// // $total = $res->fetchArray(SQLITE3_ASSOC);
// // echo "Total records: " . $total['total'] . "\n";
